@@ -40,10 +40,11 @@ import net.sourceforge.jnlp.runtime.JNLPRuntime;
 import net.sourceforge.jnlp.services.InstanceExistsException;
 import net.sourceforge.jnlp.services.ServiceUtil;
 
-import javax.swing.SwingUtilities;
 import javax.swing.text.html.parser.ParserDelegator;
 import net.sourceforge.jnlp.splashscreen.SplashUtils;
+import net.sourceforge.jnlp.util.StreamUtils;
 import net.sourceforge.jnlp.util.logging.OutputController;
+import net.sourceforge.swing.SwingUtils;
 
 import sun.awt.SunToolkit;
 
@@ -82,12 +83,14 @@ public class Launcher {
     private ParserSettings parserSettings = new ParserSettings();
 
     private Map<String, List<String>> extra = null;
+    
+    public static final String KEY_JAVAWS_LOCATION = "icedtea-web.bin.location";
 
     /**
      * Create a launcher with the runtime's default update policy
      * and launch handler.
      */
-    public Launcher() {
+    private Launcher() {
         this(null, null);
 
         if (handler == null) {
@@ -117,7 +120,7 @@ public class Launcher {
      *
      * @param handler the handler to use or null for no handler.
      */
-    public Launcher(LaunchHandler handler) {
+    private Launcher(LaunchHandler handler) {
         this(handler, null);
     }
 
@@ -128,7 +131,7 @@ public class Launcher {
      * @param handler the handler to use or null for no handler.
      * @param policy the update policy to use or null for default policy.
      */
-    public Launcher(LaunchHandler handler, UpdatePolicy policy) {
+    private Launcher(LaunchHandler handler, UpdatePolicy policy) {
         if (policy == null)
             policy = JNLPRuntime.getDefaultUpdatePolicy();
 
@@ -368,9 +371,8 @@ public class Launcher {
 
   
     /**
-     * Launches the JNLP file in a new JVM instance.  The launched
-     * application's output is sent to the system out and it's
-     * standard input channel is closed.
+     * Launches the JNLP file in a new JVM instance. 
+     * All streams are properly redirected.
      *
      * @param vmArgs the arguments to pass to the new JVM. Can be empty but
      *        must not be null.
@@ -397,9 +399,8 @@ public class Launcher {
     }
 
     /**
-     * Launches the JNLP file in a new JVM instance.  The launched
-     * application's output is sent to the system out and it's
-     * standard input channel is closed.
+     * Launches the JNLP file in a new JVM instance.
+     * All streams are properly redirected.
      *
      * @param url the URL of the JNLP file to launch
      * @throws LaunchException if there was an exception
@@ -412,8 +413,7 @@ public class Launcher {
 
     /**
      * Launches the JNLP file at the specified location in a new JVM
-     * instance.  The launched application's output is sent to the
-     * system out and it's standard input channel is closed.
+     * instance. All streams are properly redirected.
      * @param vmArgs the arguments to pass to the jvm
      * @param javawsArgs the arguments to pass to javaws (aka Netx)
      * @throws LaunchException if there was an exception
@@ -421,10 +421,10 @@ public class Launcher {
     public void launchExternal(List<String> vmArgs, List<String> javawsArgs) throws LaunchException {
         try {
 
-            List<String> commands = new LinkedList<String>();
+            List<String> commands = new LinkedList<>();
 
             // this property is set by the javaws launcher to point to the javaws binary
-            String pathToWebstartBinary = System.getProperty("icedtea-web.bin.location");
+            String pathToWebstartBinary = System.getProperty(KEY_JAVAWS_LOCATION);
             commands.add(pathToWebstartBinary);
             // use -Jargument format to pass arguments to the JVM through the launcher
             for (String arg : vmArgs) {
@@ -434,11 +434,10 @@ public class Launcher {
 
             String[] command = commands.toArray(new String[] {});
 
-            Process p = Runtime.getRuntime().exec(command);
-            new StreamEater(p.getErrorStream()).start();
-            new StreamEater(p.getInputStream()).start();
-            p.getOutputStream().close();
-
+            ProcessBuilder pb = new ProcessBuilder(command);
+            pb.inheritIO();
+            Process p =pb.start();
+            StreamUtils.waitForSafely(p);
         } catch (NullPointerException ex) {
             throw launchError(new LaunchException(null, null, R("LSFatal"), R("LCExternalLaunch"), R("LNetxJarMissing"), R("LNetxJarMissingInfo")));
         } catch (Exception ex) {
@@ -552,12 +551,15 @@ public class Launcher {
                         R("LCantDetermineMainClassInfo")));
             }
 
+            OutputController.getLogger().log(OutputController.Level.ERROR_ALL, "Starting application [" + mainName + "] ...");
+            
             Class<?> mainClass = app.getClassLoader().loadClass(mainName);
 
             Method main = mainClass.getMethod("main", new Class<?>[] { String[].class });
             String args[] = file.getApplication().getArguments();
 
-            SwingUtilities.invokeAndWait(new Runnable() {
+            // create EDT within application context:
+            SwingUtils.callOnAppContext(new Runnable() {
                 // dummy method to force Event Dispatch Thread creation
                 @Override
                 public void run() {
@@ -745,6 +747,15 @@ public class Launcher {
                  appletInstance = new AppletInstance(file, group, loader, null, cont);
              }
 
+             /**
+              * Due to PR2968, moved to earlier phase, so early stages of appelt
+              * can access Thread.currentThread().getContextClassLoader().
+              *
+              * However it is notable, that init and start still do not have access to right classloader.
+              * See LoadResources test.
+              */
+             setContextClassLoaderForAllThreads(appletInstance.getThreadGroup(), appletInstance.getClassLoader());
+
             loader.setApplication(appletInstance);
 
             // Initialize applet now that ServiceManager has access to its
@@ -757,8 +768,6 @@ public class Launcher {
             appletInstance.setApplet(applet);
             appletInstance.getAppletEnvironment().setApplet(applet);
             
-            setContextClassLoaderForAllThreads(appletInstance.getThreadGroup(), appletInstance.getClassLoader());
-
             return appletInstance;
         } catch (Exception ex) {
             throw launchError(new LaunchException(file, ex, R("LSFatal"), R("LCInit"), R("LInitApplet"), R("LInitAppletInfo")), appletInstance);
