@@ -1,4 +1,4 @@
-/* 
+/*
    Copyright (C) 2008 Red Hat, Inc.
 
 This file is part of IcedTea.
@@ -36,11 +36,13 @@ exception statement from your version.
 */
 package net.sourceforge.jnlp;
 
+import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
 
 import static net.sourceforge.jnlp.runtime.Translator.R;
+import net.sourceforge.jnlp.util.docprovider.TextsProvider;
 
 public class OptionsDefinitions {
 
@@ -51,7 +53,8 @@ public class OptionsDefinitions {
         //javaws control-options
         ABOUT("-about", "BOAbout"),
         VIEWER("-viewer", "BOViewer"),
-        CLEARCACHE("-Xclearcache", "BXclearcache"),
+        CLEARCACHE("-Xclearcache", "BXclearcache", NumberOfArguments.NONE_OR_ONE),
+        LISTCACHEIDS("-Xcacheids", "BXcacheids", NumberOfArguments.NONE_OR_ONE),
         LICENSE("-license", "BOLicense"),
         HELP1("-help", "BOHelp1"),
         //javaws run-options
@@ -74,6 +77,7 @@ public class OptionsDefinitions {
         TRUSTNONE("-Xtrustnone","BOTrustnone"),
         JNLP("-jnlp","BOJnlp", NumberOfArguments.ONE),
         HTML("-html","BOHtml", NumberOfArguments.ONE_OR_MORE),
+        BROWSER("-browser", "BrowserArg", NumberOfArguments.ONE_OR_MORE),
         //itweb settings
         LIST("-list", "IBOList"),
         GET("-get", "name", "IBOGet", NumberOfArguments.ONE_OR_MORE),
@@ -87,8 +91,9 @@ public class OptionsDefinitions {
         //-help
         FILE("-file", "policy_file", "PBOFile", NumberOfArguments.ONE),
         DEFAULTFILE("-defaultfile", "PBODefaultFile"),
-        CODEBASE("-codebase", "url", "PBOCodebase", NumberOfArguments.ONE_OR_MORE),
-        ;
+        CODEBASE("-codebase", "url", "PBOCodebase", NumberOfArguments.ONE),
+        SIGNEDBY("-signedby", "certificate_alias", "PBOSignedBy", NumberOfArguments.ONE),
+        PRINCIPALS("-principals", "class_name principal_name", "PBOPrincipals", NumberOfArguments.EVEN_NUMBER_SUPPORTS_EQUALS_CHAR);
 
         public final String option;
 
@@ -144,6 +149,7 @@ public class OptionsDefinitions {
         NONE("NOAnone"),
         ONE("NOAone"),
         ONE_OR_MORE("NOAonemore"),
+        NONE_OR_ONE("NOAnonorone"),
         EVEN_NUMBER_SUPPORTS_EQUALS_CHAR("NOAevennumber");
 
         String messageKey;
@@ -175,9 +181,11 @@ public class OptionsDefinitions {
     public static List<OPTIONS> getPolicyEditorOptions() {
         return Arrays.asList(new OPTIONS[]{
             OPTIONS.HELP1,
-            OPTIONS.DEFAULTFILE,
             OPTIONS.FILE,
+            OPTIONS.DEFAULTFILE,
             OPTIONS.CODEBASE,
+            OPTIONS.SIGNEDBY,
+            OPTIONS.PRINCIPALS,
             OPTIONS.VERBOSE
             }
         );
@@ -188,6 +196,7 @@ public class OptionsDefinitions {
             OPTIONS.ABOUT,
             OPTIONS.VIEWER,
             OPTIONS.CLEARCACHE,
+            OPTIONS.LISTCACHEIDS,
             OPTIONS.LICENSE,
             OPTIONS.HELP1}
         );
@@ -212,7 +221,8 @@ public class OptionsDefinitions {
             OPTIONS.OFFLINE,
             OPTIONS.TRUSTNONE,
             OPTIONS.JNLP,
-            OPTIONS.HTML
+            OPTIONS.HTML,
+            OPTIONS.BROWSER
         });
     }
 
@@ -226,5 +236,28 @@ public class OptionsDefinitions {
         return l;
     }
 
-    
+    public static void main(String[] args) throws IOException {
+        switch (args[0]) {
+            case TextsProvider.JAVAWS:
+                printOptions(getJavaWsOptions());
+                break;
+            case TextsProvider.ITWEB_SETTINGS:
+                printOptions(getItwsettingsCommands());
+                break;
+            case TextsProvider.POLICY_EDITOR:
+                printOptions(getPolicyEditorOptions());
+                break;
+            default:
+                break;
+        }
+    }
+
+    private static void printOptions(List<OPTIONS> options) {
+        StringBuilder sb = new StringBuilder();
+        for (OPTIONS option : options) {
+            sb.append(option.option).append(" ");
+        }
+        System.out.println(sb.toString().trim());
+    }
+
 }
