@@ -16,14 +16,18 @@
 
 package net.sourceforge.jnlp.cache;
 
-import net.sourceforge.jnlp.util.logging.OutputController;
-import java.io.*;
-import java.net.*;
-import java.util.*;
+import java.io.File;
+import java.net.URL;
+import java.util.Collection;
+import java.util.EnumSet;
+import java.util.List;
+import java.util.Set;
 
-import net.sourceforge.jnlp.*;
-import net.sourceforge.jnlp.runtime.*;
-import net.sourceforge.jnlp.util.*;
+import net.sourceforge.jnlp.DownloadOptions;
+import net.sourceforge.jnlp.Version;
+import net.sourceforge.jnlp.runtime.JNLPRuntime;
+import net.sourceforge.jnlp.util.UrlUtils;
+import net.sourceforge.jnlp.util.WeakList;
 
 /**
  * <p>
@@ -45,55 +49,57 @@ import net.sourceforge.jnlp.util.*;
  * @version $Revision: 1.9 $
  */
 public class Resource {
-
     // todo: fix resources to handle different versions
 
     // todo: IIRC, any resource is checked for being up-to-date
     // only once, regardless of UpdatePolicy.  verify and fix.
 
-    /** status bits */
-    public static final int UNINITIALIZED = 0;
-    public static final int CONNECT = 1;
-    public static final int CONNECTING = 2;
-    public static final int CONNECTED = 4;
-    public static final int DOWNLOAD = 8;
-    public static final int DOWNLOADING = 16;
-    public static final int DOWNLOADED = 32;
-    public static final int ERROR = 64;
-    public static final int STARTED = 128; // enqueued or being worked on
+    public enum Status {
+        PRECONNECT,
+        CONNECTING,
+        CONNECTED,
+        PREDOWNLOAD,
+        DOWNLOADING,
+        DOWNLOADED,
+        ERROR,
+        PROCESSING // in queue or being worked on
+    }
 
     /** list of weak references of resources currently in use */
-    private static WeakList<Resource> resources = new WeakList<Resource>();
+    private static final WeakList<Resource> resources = new WeakList<>();
 
     /** weak list of trackers monitoring this resource */
-    private WeakList<ResourceTracker> trackers = new WeakList<ResourceTracker>();
+    private final WeakList<ResourceTracker> trackers = new WeakList<>();
 
     /** the remote location of the resource */
-    URL location;
+    private final URL location;
 
     /** the location to use when downloading */
     private URL downloadLocation;
 
     /** the local file downloaded to */
-    File localFile;
+    private File localFile;
 
     /** the requested version */
-    Version requestVersion;
+    private final Version requestVersion;
 
     /** the version downloaded from server */
-    Version downloadVersion;
+    private Version downloadVersion;
 
     /** amount in bytes transferred */
-    long transferred = 0;
+    private volatile long transferred = 0;
 
     /** total size of the resource, or -1 if unknown */
-    long size = -1;
+    private volatile long size = -1;
 
     /** the status of the resource */
-    int status = UNINITIALIZED;
-
+    private final EnumSet<Status> status = EnumSet.noneOf(Status.class);
+    
     /** Update policy for this resource */
-    UpdatePolicy updatePolicy;
+    private final UpdatePolicy updatePolicy;
+
+    /** Download options for this resource */
+    private DownloadOptions downloadOptions;
 
     /**
      * Create a resource.
@@ -108,8 +114,13 @@ public class Resource {
     /**
      * Return a shared Resource object representing the given
      * location and version.
+     * @param location final location of resource
+     * @param requestVersion final version of resource
+     * @param updatePolicy final policy for updating
+     * @return new resource, which is already added in resources list
      */
     public static Resource getResource(URL location, Version requestVersion, UpdatePolicy updatePolicy) {
+        //TODO -rename to create resource?
         synchronized (resources) {
             Resource resource = new Resource(location, requestVersion, updatePolicy);
 
@@ -118,8 +129,9 @@ public class Resource {
             int index = resources.indexOf(resource);
             if (index >= 0) { // return existing object
                 Resource result = resources.get(index);
-                if (result != null)
+                if (result != null) {
                     return result;
+                }
             }
 
             resources.add(resource);
@@ -131,6 +143,7 @@ public class Resource {
 
     /**
      * Returns the remote location of the resource.
+     * @return the same location as the one with which this resource was created
      */
     public URL getLocation() {
         return location;
@@ -148,10 +161,10 @@ public class Resource {
 
     /**
      * Set the url to use for downloading the resource
-     * @param location
+     * @param downloadLocation url to be donloaded
      */
-    public void setDownloadLocation(URL location) {
-        downloadLocation = location;
+    public void setDownloadLocation(URL downloadLocation) {
+        this.downloadLocation = downloadLocation;
     }
 
     /**
@@ -161,27 +174,122 @@ public class Resource {
     ResourceTracker getTracker() {
         synchronized (trackers) {
             List<ResourceTracker> t = trackers.hardList();
-            if (t.size() > 0)
+            if (t.size() > 0) {
                 return t.get(0);
+            }
 
             return null;
         }
     }
-
+    
     /**
-     * Returns true if any of the specified flags are set.
+     * @return the local file currently being downloaded
      */
-    public boolean isSet(int flag) {
-        if (flag == UNINITIALIZED)
-            return status == UNINITIALIZED;
-        else
-            return (status & flag) != 0;
+    public File getLocalFile() {
+    	return localFile;
+    }
+    
+    /**
+     * Sets the local file to be downloaded
+     * @param localFile location of stored resource
+     */
+    public void setLocalFile(File localFile) {
+    	this.localFile = localFile;
+    }
+    
+    /**
+     * @return the requested version
+     */
+    public Version getRequestVersion() {
+    	return requestVersion;
+    }
+    
+    /**
+     * @return the version downloaded from server
+     */
+    public Version getDownloadVersion() {
+    	return downloadVersion;
+    }
+    
+    /**
+     * Sets the version downloaded from server
+     * @param downloadVersion version of downloaded resource
+     */
+    public void setDownloadVersion(Version downloadVersion) {
+    	this.downloadVersion = downloadVersion;
+    }
+    
+    /**
+     * @return the amount in bytes transferred
+     */
+    public long getTransferred() {
+    	return transferred;
+    }
+    
+    /**
+     * Sets the amount transferred
+     * @param transferred set the whole transfered amount to this value
+     */
+    public void setTransferred(long transferred) {
+    	this.transferred = transferred;
+    }
+    
+    /**
+     * Increments the amount transferred (in bytes)
+     * @param incTrans transfered amount in last transfer
+     */
+    public void incrementTransferred(long incTrans) {
+    	transferred += incTrans;
     }
 
     /**
-     * Returns the update policy for this resource
-     *
-     * @return The update policy
+     * Returns the size of the resource
+     * @return size of resource (-1 if unknown)
+     */
+    public long getSize() {
+    	return size;
+    }
+
+    /**
+     * Sets the size of the resource
+     * @param size desired size of resource
+     */
+    public void setSize(long size) {
+        this.size = size;
+    }
+
+    /**
+     * @return the status of the resource
+     */
+    public Set<Status> getCopyOfStatus() {
+        return EnumSet.copyOf(status);
+
+    }
+
+    /**
+     * Check if the specified flag is set.
+     * @param flag a status flag
+     * @return true iff the flag is set
+     */
+    public boolean isSet(Status flag) {
+        synchronized (status) {
+            return status.contains(flag);
+        }
+    }
+
+    /**
+     * Check if all the specified flags are set.
+     * @param flags a collection of flags
+     * @return true iff all the flags are set
+     */
+    public boolean hasFlags(Collection<Status> flags) {
+        synchronized (status) {
+            return status.containsAll(flags);
+        }
+    }
+
+    /**
+     * @return the update policy for this resource
      */
     public UpdatePolicy getUpdatePolicy() {
         return this.updatePolicy;
@@ -190,61 +298,93 @@ public class Resource {
     /**
      * Returns a human-readable status string.
      */
-    private String getStatusString(int flag) {
-        StringBuffer result = new StringBuffer();
+    private String getStatusString() {
+        StringBuilder result = new StringBuilder();
 
-        if (flag == 0)
-            result.append("<> ");
-        if ((flag & CONNECT) != 0)
-            result.append("CONNECT ");
-        if ((flag & CONNECTING) != 0)
-            result.append("CONNECTING ");
-        if ((flag & CONNECTED) != 0)
-            result.append("CONNECTED ");
-        if ((flag & DOWNLOAD) != 0)
-            result.append("DOWNLOAD ");
-        if ((flag & DOWNLOADING) != 0)
-            result.append("DOWNLOADING ");
-        if ((flag & DOWNLOADED) != 0)
-            result.append("DOWNLOADED ");
-        if ((flag & ERROR) != 0)
-            result.append("ERROR ");
-        if ((flag & STARTED) != 0)
-            result.append("STARTED ");
+        synchronized (status) {
+            if (status.isEmpty()) {
+                return "<>";
+            }
+            for (Status stat : status) {
+                result.append(stat.toString()).append(" ");
+            }
+        }
 
-        return result.deleteCharAt(result.length() - 1).toString();
+        return result.toString().trim();
     }
 
     /**
      * Changes the status by clearing the flags in the first
      * parameter and setting the flags in the second.  This method
      * is synchronized on this resource.
+     * @param clear a collection of status flags to unset
+     * @param add a collection of status flags to set
      */
-    public void changeStatus(int clear, int add) {
-        int orig = 0;
-
-        synchronized (this) {
-            orig = status;
-
-            this.status &= ~clear;
-            this.status |= add;
+    public void changeStatus(Collection<Status> clear, Collection<Status> add) {
+        synchronized (status) {
+            if (clear != null) {
+                status.removeAll(clear);
+            }
+            if (add != null) {
+                status.addAll(add);
+            }
         }
+    }
 
-           if (status != orig) {
-            OutputController.getLogger().log("Status: " + getStatusString(status));
-            if ((status & ~orig) != 0) {
-                OutputController.getLogger().log(" +(" + getStatusString(status & ~orig) + ")");
-            }
-            if ((~status & orig) != 0) {
-                OutputController.getLogger().log(" -(" + getStatusString(~status & orig) + ")");
-            }
-            OutputController.getLogger().log(" @ " + location.getPath());
+    /**
+     * Set status flag
+     * @param flag a flag to set
+     */
+    public void setStatusFlag(Status flag) {
+        synchronized (status) {
+            status.add(flag);
+        }
+    }
+
+    /**
+     * Set flags
+     * @param flags a collection of flags to set
+     */
+    public void setStatusFlags(Collection<Status> flags) {
+        synchronized (status) {
+            status.addAll(flags);
+        }
+    }
+
+    /**
+     * Unset flags
+     * @param flags a collection of flags to unset
+     */
+    public void unsetStatusFlag(Collection<Status> flags) {
+        synchronized (status) {
+            status.removeAll(flags);
+        }
+    }
+
+    /**
+     * Clear all flags
+     */
+    public void resetStatus() {
+        synchronized (status) {
+            status.clear();
+        }
+    }
+
+    /**
+     * Check if this resource has been initialized
+     * @return true iff any flags have been set
+     */
+    public boolean isInitialized() {
+        synchronized (status) {
+            return !status.isEmpty();
         }
     }
 
     /**
      * Removes the tracker to the list of trackers monitoring this
      * resource.
+     * 
+     * @param tracker tracker to be removed
      */
     public void removeTracker(ResourceTracker tracker) {
         synchronized (trackers) {
@@ -256,6 +396,7 @@ public class Resource {
     /**
      * Adds the tracker to the list of trackers monitoring this
      * resource.
+     * @param tracker to observing resource
      */
     public void addTracker(ResourceTracker tracker) {
         synchronized (trackers) {
@@ -284,6 +425,28 @@ public class Resource {
         }
     }
 
+    public void setDownloadOptions(DownloadOptions downloadOptions) {
+        this.downloadOptions = downloadOptions;
+    }
+
+    public DownloadOptions getDownloadOptions() {
+        return this.downloadOptions;
+    }
+
+    public boolean isConnectable() {
+        return JNLPRuntime.isConnectable(this.location);
+    }
+
+    @Override
+    public int hashCode() {
+        // FIXME: should probably have a better hashcode than this, but considering
+        // #equals(Object) was already defined first (without also overriding hashcode!),
+        // this is just being implemented in line with that so we don't break HashMaps,
+        // HashSets, etc
+        return location.hashCode();
+    }
+
+    @Override
     public boolean equals(Object other) {
         if (other instanceof Resource) {
             // this prevents the URL handler from looking up the IP
@@ -291,13 +454,13 @@ public class Resource {
             // time spent in synchronized addResource determining if
             // Resource is already in a tracker, and better for offline
             // mode on some OS.
-            return CacheUtil.urlEquals(location, ((Resource) other).location);
+            return UrlUtils.urlEquals(location, ((Resource) other).location);
         }
         return false;
     }
 
+    @Override
     public String toString() {
-        return "location=" + location.toString() + " state=" + getStatusString(status);
+        return "location=" + location.toString() + " state=" + getStatusString();
     }
-
 }
