@@ -10,6 +10,7 @@ import static net.sourceforge.jnlp.cache.Resource.Status.PREDOWNLOAD;
 
 import java.io.BufferedInputStream;
 import java.io.BufferedOutputStream;
+import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
@@ -28,10 +29,15 @@ import java.util.jar.Pack200;
 import java.util.zip.GZIPInputStream;
 
 import net.sourceforge.jnlp.DownloadOptions;
+import net.sourceforge.jnlp.OptionsDefinitions;
 import net.sourceforge.jnlp.Version;
+import net.sourceforge.jnlp.runtime.Boot;
 import net.sourceforge.jnlp.runtime.JNLPRuntime;
 import net.sourceforge.jnlp.security.ConnectionFactory;
+import net.sourceforge.jnlp.security.SecurityDialogs;
+import net.sourceforge.jnlp.security.dialogs.InetSecurity511Panel;
 import net.sourceforge.jnlp.util.HttpUtils;
+import net.sourceforge.jnlp.util.UrlUtils;
 import net.sourceforge.jnlp.util.logging.OutputController;
 
 public class ResourceDownloader implements Runnable {
@@ -156,7 +162,6 @@ public class ResourceDownloader implements Runnable {
             if (lm == null) {
                 lm = connection.getLastModified();
             }
-
             boolean current = CacheUtil.isCurrent(resource.getLocation(), resource.getRequestVersion(), lm) && resource.getUpdatePolicy() != UpdatePolicy.FORCE;
             if (!current) {
                 if (entry.isCached()) {
@@ -188,8 +193,33 @@ public class ResourceDownloader implements Runnable {
                 entry.setRemoteContentLength(size);
                 entry.setLastModified(lm);
             }
-
             entry.setLastUpdated(System.currentTimeMillis());
+            try { 
+                //do not die here no metter of cost. Just metadata
+                //is the path from user best to store? He can run some jnlp from temp which then be stored
+                //on contrary, this downloads the jnlp, we actually do not have jnlp parsed during first interaction
+                //in addition, downloaded name can be really nasty (some generated has from dynamic servlet.jnlp)
+                //anjother issue is forking. If this (eg local) jnlp starts its second isntance, the url *can* be different
+                //in contrary, usally si no. as fork is reusing all args, and only adding xmx/xms and xnofork.
+                String jnlpPath = Boot.getOptionParser().getMainArg(); //get jnlp from args passed 
+                if (jnlpPath == null || jnlpPath.equals("")) {
+                    jnlpPath = Boot.getOptionParser().getParam(OptionsDefinitions.OPTIONS.JNLP);
+                    if (jnlpPath == null || jnlpPath.equals("")) {
+                        jnlpPath = Boot.getOptionParser().getParam(OptionsDefinitions.OPTIONS.HTML);
+                        if (jnlpPath == null || jnlpPath.equals("")) {
+                            OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "Not-setting jnlp-path for missing main/jnlp/html argument");
+                        } else {
+                            entry.setJnlpPath(jnlpPath);
+                        }
+                    } else {
+                        entry.setJnlpPath(jnlpPath);
+                    }
+                } else {
+                    entry.setJnlpPath(jnlpPath);
+                }
+            } catch (Exception ex){
+                OutputController.getLogger().log(OutputController.Level.ERROR_ALL, ex);
+            }
             entry.store();
 
             synchronized (lock) {
@@ -261,6 +291,18 @@ public class ResourceDownloader implements Runnable {
                     requestProperties.put("Accept-Encoding", "pack200-gzip, gzip");
 
                     UrlRequestResult response = getUrlResponseCodeWithRedirectonResult(url, requestProperties, requestMethod);
+                    if (response.result == 511) {
+                        if (!InetSecurity511Panel.isSkip()) {
+
+                            boolean result511 = SecurityDialogs.show511Dialogue(resource);
+                            if (!result511) {
+                                throw new RuntimeException("Terminated on users request after encauntering 'http 511 authentication'.");
+                            }
+                            //try again, what to do with original resource was nowhere specified
+                            i--;
+                            continue;
+                        }
+                    }
                     if (response.shouldRedirect()) {
                         if (response.URL == null) {
                             OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "Although " + resource.toString() + " got redirect " + response.result + " code for " + requestMethod + " request for " + url.toExternalForm() + " the target was null. Not following");
@@ -366,7 +408,24 @@ public class ResourceDownloader implements Runnable {
         CacheEntry downloadEntry = new CacheEntry(downloadLocation, resource.getDownloadVersion());
         OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "Downloading file: " + downloadLocation + " into: " + downloadEntry.getCacheFile().getCanonicalPath());
         if (!downloadEntry.isCurrent(connection.getLastModified())) {
-            writeDownloadToFile(resource, downloadLocation, new BufferedInputStream(connection.getInputStream()));
+            try {
+                writeDownloadToFile(resource, downloadLocation, new BufferedInputStream(connection.getInputStream()));
+            } catch (IOException ex) {
+                String IH = "Invalid Http response";
+                if (ex.getMessage().equals(IH)) {
+                    OutputController.getLogger().log(ex);
+                    OutputController.getLogger().log(OutputController.Level.MESSAGE_ALL, "'" + IH + "' message detected. Attempting direct socket");
+                    Object[] result = UrlUtils.loadUrlWithInvalidHeaderBytes(connection.getURL());
+                    OutputController.getLogger().log("Header of: " + connection.getURL() + " (" + downloadLocation + ")");
+                    String head = (String) result[0];
+                    byte[] body = (byte[]) result[1];
+                    OutputController.getLogger().log(head);
+                    OutputController.getLogger().log("Body is: " + body.length + " bytes long");
+                    writeDownloadToFile(resource, downloadLocation, new ByteArrayInputStream(body));
+                } else {
+                    throw ex;
+                }
+            }
         } else {
             resource.setTransferred(CacheUtil.getCacheFile(downloadLocation, resource.getDownloadVersion()).length());
         }
@@ -450,6 +509,7 @@ public class ResourceDownloader implements Runnable {
      */
     static class UrlRequestResult {
 
+        //http response code
         int result = HttpURLConnection.HTTP_OK;
         URL URL;
 
