@@ -48,11 +48,17 @@ import net.sourceforge.jnlp.JARDesc;
 import net.sourceforge.jnlp.JNLPFile;
 import net.sourceforge.jnlp.LaunchException;
 import net.sourceforge.jnlp.PluginBridge;
-import net.sourceforge.jnlp.runtime.JNLPClassLoader.SecurityDelegate;
 import net.sourceforge.jnlp.runtime.JNLPRuntime;
-import net.sourceforge.jnlp.security.dialogs.apptrustwarningpanel.AppTrustWarningPanel.AppSigningWarningAction;
+import net.sourceforge.jnlp.runtime.JNLPClassLoader.SecurityDelegate;
 import net.sourceforge.jnlp.security.CertVerifier;
 import net.sourceforge.jnlp.security.SecurityDialogs;
+import net.sourceforge.jnlp.security.dialogresults.BasicDialogValue;
+import net.sourceforge.jnlp.security.dialogresults.YesNo;
+import net.sourceforge.jnlp.security.dialogresults.YesNoSandbox;
+import net.sourceforge.jnlp.security.dialogs.remember.SavedRememberAction;
+import net.sourceforge.jnlp.security.dialogs.remember.AppletSecurityActions;
+import net.sourceforge.jnlp.security.dialogs.remember.ExecuteAppletAction;
+import net.sourceforge.jnlp.security.dialogs.remember.RememberableDialog;
 import net.sourceforge.jnlp.util.UrlUtils;
 import net.sourceforge.jnlp.util.logging.OutputController;
 
@@ -81,7 +87,7 @@ public class UnsignedAppletTrustConfirmation {
      * @param id of wonted  action
      * @return the remembered decision
      */
-    public static UnsignedAppletActionEntry getStoredEntry(JNLPFile file, int id) {
+    public static UnsignedAppletActionEntry getStoredEntry(JNLPFile file, Class<? extends RememberableDialog> id) {
         UnsignedAppletActionStorage userActionStorage = securitySettings.getUnsignedAppletActionCustomStorage();
         UnsignedAppletActionStorage globalActionStorage = securitySettings.getUnsignedAppletActionGlobalStorage();
 
@@ -99,7 +105,7 @@ public class UnsignedAppletTrustConfirmation {
             return userEntry;
         }
     }
-    public static ExecuteAppletAction getStoredAction(JNLPFile file, int id) {
+    public static ExecuteAppletAction getStoredAction(JNLPFile file, Class<? extends RememberableDialog> id) {
         UnsignedAppletActionEntry x = getStoredEntry(file, id);
         if (x != null) {
             return x.getAppletSecurityActions().getAction(id);
@@ -107,7 +113,7 @@ public class UnsignedAppletTrustConfirmation {
         return null;
     }
 
-    private static UnsignedAppletActionEntry getMatchingItem(UnsignedAppletActionStorage actionStorage, JNLPFile file, int id) {
+    private static UnsignedAppletActionEntry getMatchingItem(UnsignedAppletActionStorage actionStorage, JNLPFile file, Class<? extends RememberableDialog> id) {
         return actionStorage.getMatchingItem(
                 UrlUtils.normalizeUrlAndStripParams(file.getSourceLocation(), true /* encode local files */).toString(), 
                 UrlUtils.normalizeUrlAndStripParams(file.getNotNullProbalbeCodeBase(), true /* encode local files */).toString(), 
@@ -127,7 +133,7 @@ public class UnsignedAppletTrustConfirmation {
         return fileNames;
     }
 
-    public static void updateAppletAction(JNLPFile file, ExecuteAppletAction behaviour, boolean rememberForCodeBase, int id) {
+    public static void updateAppletAction(JNLPFile file, SavedRememberAction behaviour, Boolean rememberForCodeBase, Class<? extends RememberableDialog> id) {
         UnsignedAppletActionStorage userActionStorage = securitySettings.getUnsignedAppletActionCustomStorage();
 
         userActionStorage.lock(); // We should ensure this operation is atomic
@@ -137,27 +143,38 @@ public class UnsignedAppletTrustConfirmation {
             URL codebase = UrlUtils.normalizeUrlAndStripParams(file.getNotNullProbalbeCodeBase(), true /* encode local files */);
             URL documentbase = UrlUtils.normalizeUrlAndStripParams(file.getSourceLocation(), true /* encode local files */);
 
-            /* Else, create a new entry */
-            UrlRegEx codebaseRegex = UrlRegEx.quote(codebase.toExternalForm());
-            UrlRegEx documentbaseRegex = UrlRegEx.quoteAndStar(UrlUtils.stripFile(documentbase)); // Match any from codebase and sourceFile "base"
-            List<String> archiveMatches = null; // Match any from codebase
+            UrlRegEx codebaseRegex = null;
+            UrlRegEx documentbaseRegex = null;
+            List<String> archiveMatches = null;
+            
+            if (rememberForCodeBase != null) {
+                
+                codebaseRegex = UrlRegEx.quote(codebase.toExternalForm());
 
-            if (!rememberForCodeBase) { 
-                documentbaseRegex = UrlRegEx.quote(documentbase.toExternalForm()); // Match only this applet
-                archiveMatches = toRelativePaths(getJars(file), file.getNotNullProbalbeCodeBase().toString()); // Match only this applet
+                if (!rememberForCodeBase) {
+                    documentbaseRegex = UrlRegEx.quote(documentbase.toExternalForm()); // Match only this applet
+                    archiveMatches = toRelativePaths(getJars(file), file.getNotNullProbalbeCodeBase().toString()); // Match only this applet
+                } else {
+                    documentbaseRegex = UrlRegEx.quoteAndStar(UrlUtils.stripFile(documentbase)); // Match any from codebase and sourceFile "base"
+                }
             }
             
             /* Update, if entry exists */
             if (oldEntry != null) {
                 oldEntry.getAppletSecurityActions().setAction(id, behaviour);
                 oldEntry.setTimeStamp(new Date());
-                oldEntry.setDocumentBase(documentbaseRegex);
-                oldEntry.setCodeBase(codebaseRegex);
+                if (rememberForCodeBase != null) {
+                    oldEntry.setDocumentBase(documentbaseRegex);
+                    oldEntry.setCodeBase(codebaseRegex);
+                }
                 oldEntry.setArchives(archiveMatches);
                 userActionStorage.update(oldEntry);
                 return;
             }
-
+              if (rememberForCodeBase == null){
+                  throw new RuntimeException("Trying to create new entry without codebase. Thats forbidden.");
+              } 
+                              /* Else, create a new entry */
             UnsignedAppletActionEntry entry = new UnsignedAppletActionEntry(
                     AppletSecurityActions.fromAction(id, behaviour), 
                     new Date(),
@@ -177,7 +194,7 @@ public class UnsignedAppletTrustConfirmation {
             return ((PluginBridge) file).getArchiveJars();
 
         List<JARDesc> jars = Arrays.asList(file.getResources().getJARs());
-        List<String> result = new ArrayList<String>();
+        List<String> result = new ArrayList<>();
         for (JARDesc jar : jars) {
             result.add(jar.getLocation().toString());
         }
@@ -186,40 +203,21 @@ public class UnsignedAppletTrustConfirmation {
 
     public static void checkUnsignedWithUserIfRequired(JNLPFile file) throws LaunchException {
 
-        if (unsignedAppletsAreForbidden() || JNLPRuntime.isTrustNone()) {
+        if (unsignedAppletsAreForbidden()) {
             OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "Not running unsigned applet at " + file.getCodeBase() +" because unsigned applets are disallowed by security policy.");
             throw new LaunchException(file, null, R("LSFatal"), R("LCClient"), R("LUnsignedApplet"), R("LUnsignedAppletPolicyDenied"));
         }
 
-        if (!unsignedConfirmationIsRequired() || JNLPRuntime.isTrustAll()) {
+        if (!unsignedConfirmationIsRequired()) {
             OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "Running unsigned applet at " + file.getCodeBase() +" does not require confirmation according to security policy.");
             return;
         }
 
-        ExecuteAppletAction storedAction = getStoredAction(file, AppletSecurityActions.UNSIGNED_APPLET_ACTION);
-        OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "Stored action for unsigned applet at " + file.getCodeBase() +" was " + storedAction);
+        YesNo warningResponse = SecurityDialogs.showUnsignedWarningDialog(file);
 
-        boolean appletOK;
+        OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "Decided action for unsigned applet at " + file.getCodeBase() + " was " + warningResponse);
 
-        if (storedAction == ExecuteAppletAction.ALWAYS) {
-            appletOK = true;
-        } else if (storedAction == ExecuteAppletAction.NEVER) {
-            appletOK = false;
-        } else {
-            // No remembered decision, prompt the user
-            AppSigningWarningAction warningResponse = SecurityDialogs.showUnsignedWarningDialog(file);
-            ExecuteAppletAction executeAction = warningResponse.getAction();
-
-            appletOK = (executeAction == ExecuteAppletAction.YES || executeAction == ExecuteAppletAction.ALWAYS);
-
-            if (executeAction != null) {
-                updateAppletAction(file, executeAction, warningResponse.rememberForCodeBase(), AppletSecurityActions.UNSIGNED_APPLET_ACTION);
-            }
-
-            OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "Decided action for unsigned applet at " + file.getCodeBase() +" was " + executeAction);
-        }
-
-        if (!appletOK) {
+        if (warningResponse == null || !warningResponse.compareValue(BasicDialogValue.Primitive.YES)) {
             throw new LaunchException(file, null, R("LSFatal"), R("LCClient"), R("LUnsignedApplet"), R("LUnsignedAppletUserDenied"));
         }
 
@@ -228,52 +226,23 @@ public class UnsignedAppletTrustConfirmation {
     public static void checkPartiallySignedWithUserIfRequired(SecurityDelegate securityDelegate, JNLPFile file,
             CertVerifier certVerifier) throws LaunchException {
 
-        if (JNLPRuntime.isTrustNone()) {
-            OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "Running partially signed applet at " + file.getCodeBase() + " with only Sandbox permissions due to -Xtrustnone flag");
-            securityDelegate.setRunInSandbox();
-            return;
-        }
-        if (JNLPRuntime.isTrustAll()) {
-            OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "Running partially signed applet at " + file.getCodeBase() + " due to -Xtrustall flag");
-            return;
-        }
-
         if (!unsignedConfirmationIsRequired()) {
             OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "Running partially signed applet at " + file.getCodeBase() + " does not require confirmation according to security policy.");
             return;
         }
 
-        ExecuteAppletAction storedAction = getStoredAction(file, AppletSecurityActions.UNSIGNED_APPLET_ACTION);
-        OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "Stored action for partially signed applet at " + file.getCodeBase() + " was " + storedAction);
+        YesNoSandbox warningResponse = SecurityDialogs.showPartiallySignedWarningDialog(file, certVerifier, securityDelegate);
 
-        boolean appletOK;
-
-        if (storedAction == ExecuteAppletAction.ALWAYS) {
-            appletOK = true;
-        } else if (storedAction == ExecuteAppletAction.NEVER) {
-            appletOK = false;
-        } else {
-            // No remembered decision, prompt the user
-            AppSigningWarningAction warningResponse = SecurityDialogs.showPartiallySignedWarningDialog(file, certVerifier, securityDelegate);
-            ExecuteAppletAction executeAction = warningResponse.getAction();
-
-            if (executeAction == ExecuteAppletAction.SANDBOX) {
-                securityDelegate.setRunInSandbox();
-            }
-
-            appletOK = (executeAction == ExecuteAppletAction.YES || executeAction == ExecuteAppletAction.ALWAYS
-                    || executeAction == ExecuteAppletAction.SANDBOX);
-
-            if (executeAction != null) {
-                updateAppletAction(file, executeAction, warningResponse.rememberForCodeBase(),AppletSecurityActions.UNSIGNED_APPLET_ACTION);
-            }
-
-            OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "Decided action for unsigned applet at " + file.getCodeBase() + " was " + executeAction);
-        }
-
-        if (!appletOK) {
+        OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "Decided action for unsigned applet at " + file.getCodeBase() + " was " + warningResponse);
+        
+        if (warningResponse == null || warningResponse.compareValue(BasicDialogValue.Primitive.NO)) {
             throw new LaunchException(file, null, R("LSFatal"), R("LCClient"), R("LPartiallySignedApplet"), R("LPartiallySignedAppletUserDenied"));
         }
+        
+        //this is due to possible YesNoSandboxLimited
+        if (YesNoSandbox.sandbox().compareValue(warningResponse)) {
+            securityDelegate.setRunInSandbox();
+        }
 
     }
 
