@@ -17,7 +17,6 @@
 package net.sourceforge.jnlp.util;
 
 import java.awt.Component;
-import static net.sourceforge.jnlp.runtime.Translator.R;
 
 import java.io.BufferedReader;
 import java.io.BufferedWriter;
@@ -34,20 +33,26 @@ import java.io.Reader;
 import java.io.Writer;
 import java.nio.channels.FileChannel;
 import java.nio.channels.FileLock;
+import java.nio.file.Files;
+import java.nio.file.attribute.AclEntry;
+import java.nio.file.attribute.AclEntryFlag;
+import java.nio.file.attribute.AclEntryPermission;
+import java.nio.file.attribute.AclEntryType;
+import java.nio.file.attribute.AclFileAttributeView;
 import java.security.DigestInputStream;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
-import java.util.ArrayList;
-import java.util.List;
+import java.util.*;
 
 import javax.swing.JFrame;
 import javax.swing.JOptionPane;
-import javax.swing.SwingUtilities;
+import net.sourceforge.swing.SwingUtils;
 
 import net.sourceforge.jnlp.config.DirectoryValidator;
 import net.sourceforge.jnlp.config.DirectoryValidator.DirectoryCheckResults;
 import net.sourceforge.jnlp.runtime.JNLPRuntime;
 import net.sourceforge.jnlp.util.logging.OutputController;
+import static net.sourceforge.jnlp.runtime.Translator.R;
 
 /**
  * This class contains a few file-related utility functions.
@@ -57,6 +62,10 @@ import net.sourceforge.jnlp.util.logging.OutputController;
 
 public final class FileUtils {
 
+    private static final String WIN_DRIVE_LETTER_COLON_WILDCHAR = "WINDOWS_VERY_SPECIFIC_DOUBLEDOT";
+
+    private static final List<String> WIN_ROOT_PRINCIPALS = Arrays.asList("NT AUTHORITY\\SYSTEM", "BUILTIN\\Administrators");
+
     /**
      * Indicates whether a file was successfully opened. If not, provides specific reasons
      * along with a general failure case
@@ -77,7 +86,13 @@ public final class FileUtils {
     /**
      * list of characters not allowed in filenames
      */
-    public static final char INVALID_CHARS[] = {'\\', '/', ':', '*', '?', '"', '<', '>', '|', '[', ']', '\'', ';', '=', ','};
+    public static final List<Character> INVALID_PATH = Arrays.asList(new Character[]{':', '*', '?', '"', '<', '>', '|', '[', ']', '\'', ';', '=', ','});
+    public static final List<Character> INVALID_NAME = new ArrayList<>(INVALID_PATH);
+
+    static {
+        INVALID_NAME.add(0, '\\');
+        INVALID_NAME.add(0, '/');
+    }
 
     private static final char SANITIZED_CHAR = '_';
 
@@ -85,25 +100,28 @@ public final class FileUtils {
      * Clean up a string by removing characters that can't appear in a local
      * file name.
      *
-     * @param path
-     *        the path to sanitize
+     * @param path the path to sanitize
      * @return a sanitized version of the input which is suitable for using as a
-     *         file path
+     * file path
      */
     public static String sanitizePath(String path) {
         return sanitizePath(path, SANITIZED_CHAR);
     }
 
     public static String sanitizePath(String path, char substitute) {
-
-        for (int i = 0; i < INVALID_CHARS.length; i++) {
-            if (INVALID_CHARS[i] != File.separatorChar) {
-                if (-1 != path.indexOf(INVALID_CHARS[i])) {
-                    path = path.replace(INVALID_CHARS[i], substitute);
-                }
+        //on windows, we can recieve both c:/path/ and c:\path\
+        path = path.replace("\\", "/");
+        if (JNLPRuntime.isWindows() && path.matches("^[a-zA-Z]\\:.*")) {
+            path = path.replaceFirst(":", WIN_DRIVE_LETTER_COLON_WILDCHAR);
+        }
+        for (int i = 0; i < INVALID_PATH.size(); i++) {
+            if (-1 != path.indexOf(INVALID_PATH.get(i))) {
+                path = path.replace(INVALID_PATH.get(i), substitute);
             }
         }
-
+        if (JNLPRuntime.isWindows()) {
+            path = path.replaceFirst(WIN_DRIVE_LETTER_COLON_WILDCHAR, ":");
+        }
         return path;
     }
 
@@ -120,9 +138,9 @@ public final class FileUtils {
 
     public static String sanitizeFileName(String filename, char substitute) {
 
-        for (int i = 0; i < INVALID_CHARS.length; i++) {
-            if (-1 != filename.indexOf(INVALID_CHARS[i])) {
-                filename = filename.replace(INVALID_CHARS[i], substitute);
+        for (int i = 0; i < INVALID_NAME.size(); i++) {
+            if (-1 != filename.indexOf(INVALID_NAME.get(i))) {
+                filename = filename.replace(INVALID_NAME.get(i), substitute);
             }
         }
 
@@ -230,37 +248,52 @@ public final class FileUtils {
         }
 
         if (JNLPRuntime.isWindows()) {
-            // remove all permissions
-            if (!tempFile.setExecutable(false, false)) {
-                OutputController.getLogger().log(OutputController.Level.ERROR_ALL, R("RRemoveXPermFailed", tempFile));
-            }
-            if (!tempFile.setReadable(false, false)) {
-                OutputController.getLogger().log(OutputController.Level.ERROR_ALL, R("RRemoveRPermFailed", tempFile));
-            }
-            if (!tempFile.setWritable(false, false)) {
-                OutputController.getLogger().log(OutputController.Level.ERROR_ALL, R("RRemoveWPermFailed", tempFile));
+            // prepare ACL flags
+            Set<AclEntryFlag> flags = new LinkedHashSet<>();
+            if (tempFile.isDirectory()) {
+                flags.add(AclEntryFlag.DIRECTORY_INHERIT);
+                flags.add(AclEntryFlag.FILE_INHERIT);
             }
 
-            // allow owner to read
-            if (!tempFile.setReadable(true, true)) {
-                OutputController.getLogger().log(OutputController.Level.ERROR_ALL, R("RGetRPermFailed", tempFile));
+            // prepare ACL permissions
+            Set<AclEntryPermission> permissions = new LinkedHashSet<>();
+            permissions.addAll(Arrays.asList(
+                    AclEntryPermission.READ_DATA,
+                    AclEntryPermission.READ_NAMED_ATTRS,
+                    AclEntryPermission.EXECUTE,
+                    AclEntryPermission.READ_ATTRIBUTES,
+                    AclEntryPermission.READ_ACL,
+                    AclEntryPermission.SYNCHRONIZE));
+            if (writableByOwner) {
+                permissions.addAll(Arrays.asList(
+                        AclEntryPermission.WRITE_DATA,
+                        AclEntryPermission.APPEND_DATA,
+                        AclEntryPermission.WRITE_NAMED_ATTRS,
+                        AclEntryPermission.DELETE_CHILD,
+                        AclEntryPermission.WRITE_ATTRIBUTES,
+                        AclEntryPermission.DELETE,
+                        AclEntryPermission.WRITE_ACL,
+                        AclEntryPermission.WRITE_OWNER));
             }
 
-            // allow owner to write
-            if (writableByOwner && !tempFile.setWritable(true, true)) {
-                OutputController.getLogger().log(OutputController.Level.ERROR_ALL, R("RGetWPermFailed", tempFile));
+            // filter ACL's leaving only root and owner
+            AclFileAttributeView view = Files.getFileAttributeView(tempFile.toPath(), AclFileAttributeView.class);
+            List<AclEntry> list = new ArrayList<>();
+            String owner = view.getOwner().getName();
+            for (AclEntry ae : view.getAcl()) {
+                String principalName = ae.principal().getName();
+                if (WIN_ROOT_PRINCIPALS.contains(principalName) || owner.equals(principalName)) {
+                    list.add(AclEntry.newBuilder()
+                            .setType(AclEntryType.ALLOW)
+                            .setPrincipal(ae.principal())
+                            .setPermissions(permissions)
+                            .setFlags(flags)
+                            .build());
+                }
             }
 
-            // allow owner to enter directories
-            if (isDir && !tempFile.setExecutable(true, true)) {
-                OutputController.getLogger().log(OutputController.Level.ERROR_ALL, R("RGetXPermFailed", tempFile));
-            }
-            // rename this file. Unless the file is moved/renamed, any program that
-            // opened the file right after it was created might still be able to
-            // read the data.
-            if (!tempFile.renameTo(file)) {
-                OutputController.getLogger().log(OutputController.Level.ERROR_ALL, R("RCantRename", tempFile, file));
-            }
+            // apply ACL
+            view.setAcl(list);
         } else {
         // remove all permissions
         if (!tempFile.setExecutable(false, false)) {
@@ -287,15 +320,14 @@ public final class FileUtils {
         if (isDir && !tempFile.setExecutable(true, true)) {
             throw new IOException(R("RGetXPermFailed", tempFile));
         }
-        
+        }
+
         // rename this file. Unless the file is moved/renamed, any program that
         // opened the file right after it was created might still be able to
         // read the data.
         if (!tempFile.renameTo(file)) {
             throw new IOException(R("RCantRename", tempFile, file));
         }
-        }
-
     }
 
     /**
@@ -363,7 +395,7 @@ public final class FileUtils {
      * @param frame a {@link JFrame} to act as parent to this dialog
      */
     public static void showReadOnlyDialog(final Component frame) {
-        SwingUtilities.invokeLater(new Runnable() {
+        SwingUtils.invokeLater(new Runnable() {
             @Override
             public void run() {
                 JOptionPane.showMessageDialog(frame, R("RFileReadOnly"), R("Warning"), JOptionPane.WARNING_MESSAGE);
@@ -411,7 +443,7 @@ public final class FileUtils {
      * @param message a {@link String} giving the specific reason the file could not be opened
      */
     public static void showCouldNotOpenDialog(final Component frame, final String message) {
-        SwingUtilities.invokeLater(new Runnable() {
+        SwingUtils.invokeLater(new Runnable() {
             @Override
             public void run() {
                 JOptionPane.showMessageDialog(frame, message, R("Error"), JOptionPane.ERROR_MESSAGE);
